<?php

namespace Drupal\content_moderation_reviewer\Plugin\Validation\Constraint;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\content_moderation_reviewer\AccessChecker;
use Drupal\content_moderation_reviewer\CmrConstants;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Checks if a moderation state transition is valid.
 */
class ModerationStateReviewerConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * The access checker.
   *
   * @var \Drupal\content_moderation_reviewer\AccessChecker
   */
  protected $accessChecker;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The moderation info.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInformation;

  /**
   * Creates a new ModerationStateConstraintValidator instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_information
   *   The moderation information.
   * @param \Drupal\content_moderation_reviewer\AccessChecker $access_checker
   *   The access checker.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModerationInformationInterface $moderation_information, AccessChecker $access_checker) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moderationInformation = $moderation_information;
    $this->accessChecker = $access_checker;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('content_moderation.moderation_information'),
      $container->get('content_moderation_reviewer.access_checker'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint) {
    $entity = $value->getEntity();
    assert($entity instanceof ContentEntityInterface);

    // Ignore entities that are not subject to moderation anyway.
    if (!$this->moderationInformation->isModeratedEntity($entity)) {
      return;
    }

    if ($entity->isNew()) {
      return;
    }

    if (!$entity->get(CmrConstants::FIELD_NAME)->target_id) {
      return;
    }

    $workflow = $this->moderationInformation->getWorkflowForEntity($entity);

    // If a new state is being set and there is an existing state, validate
    // there is a valid transition between them.
    $new_state = $workflow->getTypePlugin()->getState($entity->moderation_state->value);
    $original_state = $this->moderationInformation->getOriginalState($entity);
    $new_reviewer = $entity->get(CmrConstants::FIELD_NAME)->entity;
    assert(!$new_reviewer || ($new_reviewer instanceof UserInterface));

    if ($new_reviewer && !$this->accessChecker->isValidReviewer($workflow->id(), $new_reviewer, $new_state->id())) {
      $this->context->addViolation($constraint->message, [
        '%uid' => $new_reviewer->id(),
        '%from' => $original_state->label(),
        '%to' => $new_state->label(),
      ]);
    }
  }

}
